@@ -32,12 +32,18 @@
 
 #include "gupnp-root-device.h"
 #include "gupnp-context-private.h"
+#include "gupnp-error.h"
 #include "http-headers.h"
 #include "xml-util.h"
 
-G_DEFINE_TYPE (GUPnPRootDevice,
-               gupnp_root_device,
-               GUPNP_TYPE_DEVICE);
+static void
+gupnp_root_device_initable_iface_init (gpointer g_iface,
+                                       gpointer iface_data);
+
+static gboolean
+gupnp_root_device_initable_init (GInitable     *initable,
+                                 GCancellable  *cancellable,
+                                 GError       **error);
 
 struct _GUPnPRootDevicePrivate {
         GUPnPXMLDoc *description_doc;
@@ -48,6 +54,16 @@ struct _GUPnPRootDevicePrivate {
         char  *description_dir;
         char  *relative_location;
 };
+typedef struct _GUPnPRootDevicePrivate GUPnPRootDevicePrivate;
+
+G_DEFINE_TYPE_EXTENDED (GUPnPRootDevice,
+                        gupnp_root_device,
+                        GUPNP_TYPE_DEVICE,
+                        0,
+                        G_ADD_PRIVATE(GUPnPRootDevice)
+                        G_IMPLEMENT_INTERFACE
+                                (G_TYPE_INITABLE,
+                                 gupnp_root_device_initable_iface_init));
 
 enum {
         PROP_0,
@@ -61,14 +77,16 @@ static void
 gupnp_root_device_finalize (GObject *object)
 {
         GUPnPRootDevice *device;
+        GUPnPRootDevicePrivate *priv;
         GObjectClass *object_class;
 
         device = GUPNP_ROOT_DEVICE (object);
+        priv = gupnp_root_device_get_instance_private (device);
 
-        g_object_unref (device->priv->description_doc);
-        g_free (device->priv->description_path);
-        g_free (device->priv->description_dir);
-        g_free (device->priv->relative_location);
+        g_clear_object (&priv->description_doc);
+        g_free (priv->description_path);
+        g_free (priv->description_dir);
+        g_free (priv->relative_location);
 
         /* Call super */
         object_class = G_OBJECT_CLASS (gupnp_root_device_parent_class);
@@ -79,13 +97,15 @@ static void
 gupnp_root_device_dispose (GObject *object)
 {
         GUPnPRootDevice *device;
+        GUPnPRootDevicePrivate *priv;
         GObjectClass *object_class;
 
         device = GUPNP_ROOT_DEVICE (object);
+        priv = gupnp_root_device_get_instance_private (device);
 
-        if (device->priv->group) {
-                g_object_unref (device->priv->group);
-                device->priv->group = NULL;
+        if (priv->group) {
+                g_object_unref (priv->group);
+                priv->group = NULL;
         }
 
         /* Call super */
@@ -96,9 +116,14 @@ gupnp_root_device_dispose (GObject *object)
 static void
 gupnp_root_device_init (GUPnPRootDevice *device)
 {
-        device->priv = G_TYPE_INSTANCE_GET_PRIVATE (device,
-                                                    GUPNP_TYPE_ROOT_DEVICE,
-                                                    GUPnPRootDevicePrivate);
+}
+
+static void
+gupnp_root_device_initable_iface_init (gpointer g_iface,
+                                       gpointer iface_data)
+{
+        GInitableIface *iface = (GInitableIface *) g_iface;
+        iface->init = gupnp_root_device_initable_init;
 }
 
 static void
@@ -108,18 +133,20 @@ gupnp_root_device_set_property (GObject      *object,
                                 GParamSpec   *pspec)
 {
         GUPnPRootDevice *device;
+        GUPnPRootDevicePrivate *priv;
 
         device = GUPNP_ROOT_DEVICE (object);
+        priv = gupnp_root_device_get_instance_private (device);
 
         switch (property_id) {
         case PROP_DESCRIPTION_DOC:
-                device->priv->description_doc = g_value_dup_object (value);
+                priv->description_doc = g_value_dup_object (value);
                 break;
         case PROP_DESCRIPTION_PATH:
-                device->priv->description_path = g_value_dup_string (value);
+                priv->description_path = g_value_dup_string (value);
                 break;
         case PROP_DESCRIPTION_DIR:
-                device->priv->description_dir = g_value_dup_string (value);
+                priv->description_dir = g_value_dup_string (value);
                 break;
         case PROP_AVAILABLE:
                 gupnp_root_device_set_available
@@ -138,17 +165,19 @@ gupnp_root_device_get_property (GObject    *object,
                                 GParamSpec *pspec)
 {
         GUPnPRootDevice *device;
+        GUPnPRootDevicePrivate *priv;
 
         device = GUPNP_ROOT_DEVICE (object);
+        priv = gupnp_root_device_get_instance_private (device);
 
         switch (property_id) {
         case PROP_DESCRIPTION_PATH:
                 g_value_set_string (value,
-                                    device->priv->description_path);
+                                    priv->description_path);
                 break;
         case PROP_DESCRIPTION_DIR:
                 g_value_set_string (value,
-                                    device->priv->description_dir);
+                                    priv->description_dir);
                 break;
         case PROP_AVAILABLE:
                 g_value_set_boolean (value,
@@ -262,113 +291,98 @@ load_and_parse (const char *description_path)
         return description_doc;
 }
 
-static GObject *
-gupnp_root_device_constructor (GType                  type,
-                               guint                  n_construct_params,
-                               GObjectConstructParam *construct_params)
+static gboolean
+gupnp_root_device_initable_init (GInitable     *initable,
+                                 GCancellable  *cancellable,
+                                 GError       **error)
 {
-        GObjectClass *object_class;
-        GObject *object;
         GUPnPRootDevice *device;
         GUPnPContext *context;
-        const char *description_path, *description_dir, *udn;
+        const char *udn;
         SoupURI *uri;
         char *desc_path, *location, *usn, *relative_location;
-        unsigned int i;
-        GUPnPXMLDoc *description_doc;
         xmlNode *root_element, *element;
         SoupURI *url_base;
+        gboolean result = FALSE;
+        GUPnPRootDevicePrivate *priv;
+
+        device = GUPNP_ROOT_DEVICE (initable);
+        priv = gupnp_root_device_get_instance_private (device);
 
-        object = NULL;
         location = NULL;
 
-        /* Get 'description-doc', 'context', 'description-dir' and
-         * 'description-path' property values */
-        description_doc   = NULL;
-        context           = NULL;
-        description_path  = NULL;
-        description_dir   = NULL;
+        g_object_get (G_OBJECT (device),
+                      "context", &context,
+                      NULL);
 
-        for (i = 0; i < n_construct_params; i++) {
-                const char *par_name;
+        if (context == NULL) {
+                g_set_error_literal (error,
+                                     GUPNP_ROOT_DEVICE_ERROR,
+                                     GUPNP_ROOT_DEVICE_ERROR_NO_CONTEXT,
+                                     "No context specified");
 
-                par_name = construct_params[i].pspec->name;
-
-                if (strcmp (par_name, "description-doc") == 0) {
-                        description_doc =
-                                g_value_get_object (construct_params[i].value);
-
-                        continue;
-                } 
-
-                if (strcmp (par_name, "context") == 0) {
-                        context =
-                                g_value_get_object (construct_params[i].value);
-
-                        continue;
-                }
-
-                if (strcmp (par_name, "description-path") == 0) {
-                        description_path =
-                                g_value_get_string (construct_params[i].value);
-
-                        continue;
-                }
-
-                if (strcmp (par_name, "description-dir") == 0) {
-                        description_dir =
-                                g_value_get_string (construct_params[i].value);
-
-                        continue;
-                }
+                return FALSE;
         }
 
-        if (!context) {
-                g_warning ("No context specified.");
+        if (priv->description_path == NULL) {
+                g_set_error_literal (error,
+                                     GUPNP_ROOT_DEVICE_ERROR,
+                                     GUPNP_ROOT_DEVICE_ERROR_NO_DESCRIPTION_PATH,
+                                     "Path to description document not specified.");
 
-                return NULL;
+                return FALSE;
         }
 
-        if (!description_path) {
-                g_warning ("Path to description document not specified.");
+        if (priv->description_dir == NULL) {
+                g_set_error_literal (error,
+                                     GUPNP_ROOT_DEVICE_ERROR,
+                                     GUPNP_ROOT_DEVICE_ERROR_NO_DESCRIPTION_FOLDER,
+                                     "Path to description directory not specified.");
 
-                return NULL;
-        }
-
-        if (!description_dir) {
-                g_warning ("Path to description directory not specified.");
-
-                return NULL;
+                return FALSE;
         }
 
         uri = _gupnp_context_get_server_uri (context);
         if (uri == NULL) {
-                g_warning ("Network interface is not usable");
+                g_set_error_literal (error,
+                                     GUPNP_ROOT_DEVICE_ERROR,
+                                     GUPNP_ROOT_DEVICE_ERROR_NO_NETWORK,
+                                     "Network interface is not usable");
 
-                return NULL;
+                return FALSE;
         }
 
-        if (g_path_is_absolute (description_path))
-                desc_path = g_strdup (description_path);
+        if (g_path_is_absolute (priv->description_path))
+                desc_path = g_strdup (priv->description_path);
         else
-                desc_path = g_build_filename (description_dir,
-                                              description_path,
+                desc_path = g_build_filename (priv->description_dir,
+                                              priv->description_path,
                                               NULL);
 
         /* Check whether we have a parsed description document */
-        if (!description_doc) {
+        if (priv->description_doc == NULL) {
                 /* We don't, so load and parse it */
-                description_doc = load_and_parse (desc_path);
-                if (description_doc == NULL)
+                priv->description_doc = load_and_parse (desc_path);
+                if (priv->description_doc == NULL) {
+                        g_set_error_literal (error,
+                                             GUPNP_XML_ERROR,
+                                             GUPNP_XML_ERROR_PARSE,
+                                             "Coupld not parse description document");
+
                         goto DONE;
+                }
         }
 
         /* Find correct element */
-        root_element = xml_util_get_element ((xmlNode *) description_doc->doc,
+        root_element = xml_util_get_element ((xmlNode *)
+                                             gupnp_xml_doc_get_doc (priv->description_doc),
                                              "root",
                                              NULL);
         if (!root_element) {
-                g_warning ("\"/root\" element not found.");
+                g_set_error_literal (error,
+                                     GUPNP_XML_ERROR,
+                                     GUPNP_XML_ERROR_NO_NODE,
+                                     "\"/root\" element not found.");
 
                 goto DONE;
         }
@@ -377,55 +391,33 @@ gupnp_root_device_constructor (GType                  type,
                                         "device",
                                         NULL);
         if (!element) {
-                g_warning ("\"/root/device\" element not found.");
+                g_set_error_literal (error,
+                                     GUPNP_XML_ERROR,
+                                     GUPNP_XML_ERROR_NO_NODE,
+                                     "\"/root/device\" element not found.");
 
                 goto DONE;
         }
 
-        /* Set 'element' and 'description-doc' properties */
-        for (i = 0; i < n_construct_params; i++) {
-                const char *par_name;
-
-                par_name = construct_params[i].pspec->name;
-
-                if (strcmp (par_name, "element") == 0) {
-                        g_value_set_pointer (construct_params[i].value,
-                                             element);
-
-                        continue;
-                }
-
-                if (strcmp (par_name, "description-doc") == 0) {
-                        g_value_set_object (construct_params[i].value,
-                                            description_doc);
-
-                        continue;
-                }
-        }
-
-        /* Create object */
-        object_class = G_OBJECT_CLASS (gupnp_root_device_parent_class);
-
-        object = object_class->constructor (type,
-                                            n_construct_params,
-                                            construct_params);
-        device = GUPNP_ROOT_DEVICE (object);
+        g_object_set (G_OBJECT (device),
+                      "element", element,
+                      NULL);
 
         /* Generate location relative to HTTP root */
         udn = gupnp_device_info_get_udn (GUPNP_DEVICE_INFO (device));
         if (udn && strstr (udn, "uuid:") == udn)
-                device->priv->relative_location = g_strdup_printf ("%s.xml", udn + 5);
+                priv->relative_location = g_strdup_printf ("%s.xml", udn + 5);
         else
-                device->priv->relative_location = g_strdup_printf ("RootDevice%p.xml", device);
+                priv->relative_location = g_strdup_printf ("RootDevice%p.xml", device);
 
         relative_location = g_strjoin (NULL,
                                        "/",
-                                       device->priv->relative_location,
+                                       priv->relative_location,
                                        NULL);
 
         /* Host the description file and dir */
         gupnp_context_host_path (context, desc_path, relative_location);
-        gupnp_context_host_path (context, device->priv->description_dir, "");
+        gupnp_context_host_path (context, priv->description_dir, "");
 
         /* Generate full location */
         soup_uri_set_path (uri, relative_location);
@@ -441,7 +433,7 @@ gupnp_root_device_constructor (GType                  type,
                 url_base = soup_uri_new (location);
 
         /* Set additional properties */
-        g_object_set (object,
+        g_object_set (G_OBJECT (device),
                       "location", location,
                       "url-base", url_base,
                       NULL);
@@ -449,17 +441,19 @@ gupnp_root_device_constructor (GType                  type,
         soup_uri_free (url_base);
 
         /* Create resource group */
-        device->priv->group = gssdp_resource_group_new (GSSDP_CLIENT (context));
+        priv->group = gssdp_resource_group_new (GSSDP_CLIENT (context));
 
         /* Add services and devices to resource group */
         usn = g_strdup_printf ("%s::upnp:rootdevice", (const char *) udn);
-        gssdp_resource_group_add_resource_simple (device->priv->group,
+        gssdp_resource_group_add_resource_simple (priv->group,
                                                   "upnp:rootdevice",
                                                   usn,
                                                   location);
         g_free (usn);
 
-        fill_resource_group (element, location, device->priv->group);
+        fill_resource_group (element, location, priv->group);
+
+        result = TRUE;
 
  DONE:
         /* Cleanup */
@@ -469,7 +463,7 @@ gupnp_root_device_constructor (GType                  type,
         g_free (desc_path);
         g_free (location);
 
-        return object;
+        return result;
 }
 
 static void
@@ -481,12 +475,9 @@ gupnp_root_device_class_init (GUPnPRootDeviceClass *klass)
 
         object_class->set_property = gupnp_root_device_set_property;
         object_class->get_property = gupnp_root_device_get_property;
-        object_class->constructor  = gupnp_root_device_constructor;
         object_class->dispose      = gupnp_root_device_dispose;
         object_class->finalize     = gupnp_root_device_finalize;
 
-        g_type_class_add_private (klass, sizeof (GUPnPRootDevicePrivate));
-
         /**
          * GUPnPRootDevice:description-doc:
          *
@@ -571,6 +562,8 @@ gupnp_root_device_class_init (GUPnPRootDeviceClass *klass)
  * @description_path: Path to device description document. This could either
  * be an absolute path or path relative to @description_dir.
  * @description_dir: Path to directory where description documents are provided.
+ * @error: (allow-none): The location for a #GError to report issue with
+ * creation on or %NULL.
  *
  * Create a new #GUPnPRootDevice object, automatically loading and parsing
  * device description document from @description_path.
@@ -580,7 +573,8 @@ gupnp_root_device_class_init (GUPnPRootDeviceClass *klass)
 GUPnPRootDevice *
 gupnp_root_device_new (GUPnPContext *context,
                        const char   *description_path,
-                       const char   *description_dir)
+                       const char   *description_dir,
+                       GError      **error)
 {
         GUPnPResourceFactory *factory;
 
@@ -590,7 +584,8 @@ gupnp_root_device_new (GUPnPContext *context,
                                            factory,
                                            NULL,
                                            description_path,
-                                           description_dir);
+                                           description_dir,
+                                           error);
 }
 
 /**
@@ -601,6 +596,8 @@ gupnp_root_device_new (GUPnPContext *context,
  * @description_path: Path to device description document. This could either
  * be an absolute path or path relative to @description_dir.
  * @description_dir: Path to directory where description documents are provided.
+ * @error: (allow-none): The location for a #GError to report issue with
+ * creation on or %NULL.
  *
  * Create a new #GUPnPRootDevice, automatically loading and parsing
  * device description document from @description_path if @description_doc is
@@ -613,19 +610,22 @@ gupnp_root_device_new_full (GUPnPContext         *context,
                             GUPnPResourceFactory *factory,
                             GUPnPXMLDoc          *description_doc,
                             const char           *description_path,
-                            const char           *description_dir)
+                            const char           *description_dir,
+                            GError              **error)
 {
         g_return_val_if_fail (GUPNP_IS_CONTEXT (context), NULL);
         g_return_val_if_fail (GUPNP_IS_RESOURCE_FACTORY (factory), NULL);
 
-        return g_object_new (GUPNP_TYPE_ROOT_DEVICE,
-                             "context", context,
-                             "resource-factory", factory,
-                             "root-device", NULL,
-                             "description-doc", description_doc,
-                             "description-path", description_path,
-                             "description-dir", description_dir,
-                             NULL);
+        return g_initable_new (GUPNP_TYPE_ROOT_DEVICE,
+                               NULL,
+                               error,
+                               "context", context,
+                               "resource-factory", factory,
+                               "root-device", NULL,
+                               "description-doc", description_doc,
+                               "description-path", description_path,
+                               "description-dir", description_dir,
+                               NULL);
 }
 
 /**
@@ -640,10 +640,13 @@ void
 gupnp_root_device_set_available (GUPnPRootDevice *root_device,
                                  gboolean         available)
 {
+        GUPnPRootDevicePrivate *priv;
+
         g_return_if_fail (GUPNP_IS_ROOT_DEVICE (root_device));
 
-        gssdp_resource_group_set_available (root_device->priv->group,
-                                            available);
+        priv = gupnp_root_device_get_instance_private (root_device);
+
+        gssdp_resource_group_set_available (priv->group, available);
 
         g_object_notify (G_OBJECT (root_device), "available");
 }
@@ -659,9 +662,13 @@ gupnp_root_device_set_available (GUPnPRootDevice *root_device,
 gboolean
 gupnp_root_device_get_available (GUPnPRootDevice *root_device)
 {
+        GUPnPRootDevicePrivate *priv;
+
         g_return_val_if_fail (GUPNP_IS_ROOT_DEVICE (root_device), FALSE);
 
-        return gssdp_resource_group_get_available (root_device->priv->group);
+        priv = gupnp_root_device_get_instance_private (root_device);
+
+        return gssdp_resource_group_get_available (priv->group);
 }
 
 /**
@@ -675,9 +682,13 @@ gupnp_root_device_get_available (GUPnPRootDevice *root_device)
 const char *
 gupnp_root_device_get_relative_location (GUPnPRootDevice *root_device)
 {
+        GUPnPRootDevicePrivate *priv;
+
         g_return_val_if_fail (GUPNP_IS_ROOT_DEVICE (root_device), NULL);
 
-        return root_device->priv->relative_location;
+        priv = gupnp_root_device_get_instance_private (root_device);
+
+        return priv->relative_location;
 }
 
 /**
@@ -691,9 +702,13 @@ gupnp_root_device_get_relative_location (GUPnPRootDevice *root_device)
 const char *
 gupnp_root_device_get_description_path (GUPnPRootDevice *root_device)
 {
+        GUPnPRootDevicePrivate *priv;
+
         g_return_val_if_fail (GUPNP_IS_ROOT_DEVICE (root_device), NULL);
 
-        return root_device->priv->description_path;
+        priv = gupnp_root_device_get_instance_private (root_device);
+
+        return priv->description_path;
 }
 
 /**
@@ -708,9 +723,13 @@ gupnp_root_device_get_description_path (GUPnPRootDevice *root_device)
 const char *
 gupnp_root_device_get_description_dir (GUPnPRootDevice *root_device)
 {
+        GUPnPRootDevicePrivate *priv;
+
         g_return_val_if_fail (GUPNP_IS_ROOT_DEVICE (root_device), NULL);
 
-        return root_device->priv->description_dir;
+        priv = gupnp_root_device_get_instance_private (root_device);
+
+        return priv->description_dir;
 }
 
 /**
@@ -726,7 +745,11 @@ gupnp_root_device_get_description_dir (GUPnPRootDevice *root_device)
 GSSDPResourceGroup *
 gupnp_root_device_get_ssdp_resource_group (GUPnPRootDevice *root_device)
 {
+        GUPnPRootDevicePrivate *priv;
+
         g_return_val_if_fail (GUPNP_IS_ROOT_DEVICE (root_device), NULL);
 
-        return root_device->priv->group;
+        priv = gupnp_root_device_get_instance_private (root_device);
+
+        return priv->group;
 }
